import pymysql
import logging
import time
import datetime

class MySQLDB:
    """
    与mysql交互
    :param host:数据库服务器地址
    :param port:数据库服务器端口
    :param user:数据库用户名
    :param password:密码
    :param database:数据库名称
    """
    def __init__(self,host,port,user,password,database) -> None:
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.stopFlag = False
        self.logger = logging.getLogger("MysqlDAO")
        self.connect() # 初始化时自动连接
        
    
    def connect(self):
        """创建MySQL连接"""
        while not self.stopFlag:
            try:
                self.db =  pymysql.connect(host=self.host,port=self.port,user=self.user,password=self.password, database=self.database,connect_timeout=10)
                self.logger.info("数据库连接成功")
                break
            except Exception as e:
                self.logger.error(f"数据库连接失败{e}")
                time.sleep(1)

    def close(self):
        """断开MySQL连接"""
        self.db.close()
        self.stopFlag = True

    def checkConnection(self):
        """检查MySQL连接状态，若没有连接则自动重连"""
        if self.db is None or not self.db.open:
            self.logger.error("数据库连接已断开")
            self.connect()

    def saveTotalOutPut(self, TotalOutPutList):
        """
        保存总产量信息
        :param TotalOutPutList:总产量信息列表
        """
        self.checkConnection()
        timenow = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        insert_total_output_sql = """
            INSERT INTO iotmachine_total_output (all_num, ng_num, ok_num, parent_equip_no, product_name, product_no, shift_num, statistics_date,create_time,update_time,is_delete)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
        """
        query_total_output_sql = """
        SELECT * 
        FROM iotmachine_total_output 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s
        """
        update_total_output_sql = """
        UPDATE iotmachine_total_output 
        SET all_num=%s, ng_num=%s, ok_num=%s, update_time=%s 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s
        """
        try:
            cursor = self.db.cursor()
            # self.db.begin()
            for TotalOutput in TotalOutPutList :
                # 准备SQL参数
                values = (
                    TotalOutput['ParentEquipNo'],
                    TotalOutput['ProductNo'],
                    TotalOutput['ShiftNum'],
                    TotalOutput['StatisticsDate']
                )
                recoedNum = cursor.execute(query_total_output_sql,values) # 先查询是否有该记录
                self.logger.debug(recoedNum)
                if recoedNum == 0:
                    self.logger.info("新增总产量记录")
                    self.logger.debug(TotalOutput)
                    values = (
                        TotalOutput['AllNum'],
                        TotalOutput['NGNum'],
                        TotalOutput['OKNum'],
                        TotalOutput['ParentEquipNo'],
                        TotalOutput['ProductName'],
                        TotalOutput['ProductNo'],
                        TotalOutput['ShiftNum'],
                        TotalOutput['StatisticsDate'],
                        timenow,
                        timenow,
                        0
                    )
                    cursor.execute(insert_total_output_sql, values)
                else:
                    self.logger.info("更新总产量记录")
                    self.logger.debug(TotalOutput)
                    values = (
                        TotalOutput['AllNum'],
                        TotalOutput['NGNum'],
                        TotalOutput['OKNum'],
                        timenow,
                        TotalOutput['ParentEquipNo'],
                        TotalOutput['ProductNo'],
                        TotalOutput['ShiftNum'],
                        TotalOutput['StatisticsDate']
                    )
                    cursor.execute(update_total_output_sql, values)
            
            self.db.commit()
            cursor.close()
            self.logger.info("总产量记录已更新")
        except pymysql.MySQLError as e:
            self.logger.error(f"总产量记录保存失败{e}")
            # self.db.rollback()
            cursor.close()
        except Exception as e:
            self.logger.error(f"总产量记录保存失败{e}")
            cursor.close()
            self.db.close()
        

    def saveNGDetails(self, NGDetailsList):
        """
        保存ng明细信息
        :param NGDetailsList:ng明细信息列表
        """
        self.checkConnection()
        timenow = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        insert_ng_details_sql = """
            INSERT INTO iotmachine_ng_details (record_id, equip_no, ng_class_name, ng_class_no, num, parent_equip_no, product_name, product_no, shift_num, statistics_date,create_time,update_time,is_delete)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
        """
        query_total_output_sql = """
        SELECT id 
        FROM iotmachine_total_output 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s
        """
        query_ng_details_sql = """
        SELECT * 
        FROM iotmachine_ng_details 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s AND ng_class_no = %s
        """
        update_ng_details_sql = """
        UPDATE iotmachine_ng_details 
        SET num=%s, update_time=%s 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s AND ng_class_no = %s
        """
        try:
            cursor = self.db.cursor()
                # self.db.begin()
            for NGDetails in NGDetailsList :
                # 准备SQL参数
                values = (
                    NGDetails['ParentEquipNo'],
                    NGDetails['ProductNo'],
                    NGDetails['ShiftNum'],
                    NGDetails['StatisticsDate']
                )
                recoedNum = cursor.execute(query_total_output_sql,values) # 先查询主表(totaloutput)中对应的id
                if recoedNum == 0:
                    self.logger.error("NG明细没有在主表中找到记录")
                    self.logger.error(values)
                    record_id = 0
                else:
                    record_id = cursor.fetchone()[0]
                
                values = (
                    NGDetails['ParentEquipNo'],
                    NGDetails['ProductNo'],
                    NGDetails['ShiftNum'],
                    NGDetails['StatisticsDate'],
                    NGDetails['NGClassNo'],
                )
                recoedNum = cursor.execute(query_ng_details_sql,values) # 查询是否有该记录
                self.logger.debug(f"查找到记录数：{recoedNum}")
                if recoedNum == 0:
                    self.logger.info("新增NG明细记录")
                    self.logger.debug(NGDetails)
                    values = (
                        record_id,
                        NGDetails['EquipNo'],
                        NGDetails['NGClassName'],
                        NGDetails['NGClassNo'],
                        NGDetails['Num'],
                        NGDetails['ParentEquipNo'],
                        NGDetails['ProductName'],
                        NGDetails['ProductNo'],
                        NGDetails['ShiftNum'],
                        NGDetails['StatisticsDate'],
                        timenow,
                        timenow,
                        0
                    )
                    cursor.execute(insert_ng_details_sql, values)
                else:
                    self.logger.info("更新NG明细记录")
                    self.logger.debug(NGDetails)
                    values = (
                        NGDetails['Num'],
                        timenow,
                        NGDetails['ParentEquipNo'],
                        NGDetails['ProductNo'],
                        NGDetails['ShiftNum'],
                        NGDetails['StatisticsDate'],
                        NGDetails['NGClassNo'],
                    )
                    cursor.execute(update_ng_details_sql, values)
            
            self.db.commit()
            cursor.close()
            self.logger.info("NG明细记录已更新")
        except pymysql.MySQLError as e:
            self.logger.error(f"NG明细记录保存失败{e}")
            # self.db.rollback()
            cursor.close()
        except Exception as e:
            self.logger.error(f"NG明细记录保存失败{e}")
            cursor.close()
            self.db.close()

    def saveNGHourDetails(self, NGHourDetailsList):
        """
        保存ng小时分布信息
        :param NGHourDetailsList:ng小时分布信息列表
        """
        self.checkConnection()
        timenow = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        insert_ng_hour_details_sql = """
            INSERT INTO iotmachine_ng_hour_details (record_id, ng_create_date, equip_no, ng_class_name, ng_class_no, num, parent_equip_no, product_name, product_no, shift_num, statistics_date,create_time,update_time,is_delete)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
        """
        query_ng_details_sql = """
        SELECT id 
        FROM iotmachine_ng_details 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s AND ng_class_no = %s
        """
        query_ng_hour_details_sql = """
        SELECT * 
        FROM iotmachine_ng_hour_details 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s AND ng_class_no = %s AND ng_create_date = %s
        """
        update_ng_hour_details_sql = """
        UPDATE iotmachine_ng_hour_details 
        SET num=%s, update_time=%s 
        WHERE parent_equip_no=%s AND product_no=%s AND shift_num=%s AND statistics_date = %s AND ng_class_no = %s AND ng_create_date = %s
        """
        try:
            cursor = self.db.cursor()
                # self.db.begin()
            for NGHourDetails in NGHourDetailsList :
                # 准备SQL参数
                values = (
                    NGHourDetails['ParentEquipNo'],
                    NGHourDetails['ProductNo'],
                    NGHourDetails['ShiftNum'],
                    NGHourDetails['StatisticsDate'],
                    NGHourDetails["NGClassNo"]
                )
                recoedNum = cursor.execute(query_ng_details_sql,values) # 先查询主表(ngdetails)中对应的id
                if recoedNum == 0:
                    self.logger.error("NG小时分布没有在主表中找到记录")
                    self.logger.error(values)
                    record_id = 0
                else:
                    record_id = cursor.fetchone()[0]
                
                values = (
                    NGHourDetails['ParentEquipNo'],
                    NGHourDetails['ProductNo'],
                    NGHourDetails['ShiftNum'],
                    NGHourDetails['StatisticsDate'],
                    NGHourDetails['NGClassNo'],
                    NGHourDetails["CreateDate"]
                )
                recoedNum = cursor.execute(query_ng_hour_details_sql,values) # 查询是否有该记录
                self.logger.debug(f"查找到记录数：{recoedNum}")
                if recoedNum == 0:
                    self.logger.info("新增NG小时分布记录")
                    self.logger.debug(NGHourDetails)
                    values = (
                        record_id,
                        NGHourDetails['CreateDate'],
                        NGHourDetails['EquipNo'],
                        NGHourDetails['NGClassName'],
                        NGHourDetails['NGClassNo'],
                        NGHourDetails['Num'],
                        NGHourDetails['ParentEquipNo'],
                        NGHourDetails['ProductName'],
                        NGHourDetails['ProductNo'],
                        NGHourDetails['ShiftNum'],
                        NGHourDetails['StatisticsDate'],
                        timenow,
                        timenow,
                        0
                    )
                    cursor.execute(insert_ng_hour_details_sql, values)
                else:
                    self.logger.info("更新NG小时分布记录")
                    self.logger.debug(NGHourDetails)
                    values = (
                        NGHourDetails['Num'],
                        timenow,
                        NGHourDetails['ParentEquipNo'],
                        NGHourDetails['ProductNo'],
                        NGHourDetails['ShiftNum'],
                        NGHourDetails['StatisticsDate'],
                        NGHourDetails['NGClassNo'],
                        NGHourDetails['CreateDate'],
                    )
                    cursor.execute(update_ng_hour_details_sql, values)
            
            self.db.commit()
            cursor.close()
            self.logger.info("NG小时分布记录已更新")
        except pymysql.MySQLError as e:
            self.logger.error(f"NG小时分布记录保存失败{e}")
            # self.db.rollback()
            cursor.close()
        except Exception as e:
            self.logger.error(f"NG小时分布记录保存失败{e}")
            self.db.close()

